import ctypes
from .__library__ import lib, get_error
from .stream import *


class MatType:
    """
    The different matrix types
    """

    UINT8 = 0
    "A single unsigned byte"

    INT8 = 1
    "A single signed byte"

    UINT16 = 2
    "Two unsigned bytes"

    INT16 = 3
    "Two signed bytes"

    INT32 = 4
    "Four signed bytes"

    FLOAT = 5
    "Four bytes floating point value"

    DOUBLE = 6
    "Eight bytes floating point value"

    FLOAT_4 = 29
    "Four bytes floating point value with four channels"

    FLOAT_8 = 61
    "Four bytes floating point value with eight channels"



class Data():
    """
    This is the class that contains depth or image data
    """
    def __del__(self):
        if self._ptr:
            lib.tof_data_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ptr = None):
        """
        Construct new data class
        ----------

        Create new tof data

        Parameters
        ----------
        rows : int
                Number of rows
        cols : int
                Number of columns
        mat_type : MatType
                The matrix type
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        else:
            raise RuntimeError("Can't construct empty class")

    def data(self):
        """
        Get pointer to data

        Returns
        ----------
        ctypes.c_void_p
            
        """
        return lib.tof_data_data(self._ptr, get_error())


    def size(self):
        """
        Get size of the data

        Returns
        ----------
        int
            The size
        """
        return lib.tof_data_size(self._ptr, get_error())


    def getUserData(self):
        """
        Get user data that will get saved to csf file

        Returns
        ----------
        int
            Set user data
        """
        return lib.tof_data_get_user_data(self._ptr, get_error())


    def setUserData(self, user_data):
        """
        Set user data that will get saved to csf file

        Parameters
        ----------
        user_data : int
            Set user data
        """
        return lib.tof_data_set_user_data(self._ptr, user_data, get_error())


    def matType(self):
        """
        Get the matrix type

        Returns
        ----------
        MatType
            The matrix type
        """
        return lib.tof_data_mat_type(self._ptr, get_error())


    def rows(self):
        """
        Get the number of rows

        Returns
        ----------
        int
            Number of rows
        """
        return lib.tof_data_rows(self._ptr, get_error())


    def cols(self):
        """
        Get the number of columns

        Returns
        ----------
        int
            Number of columns
        """
        return lib.tof_data_cols(self._ptr, get_error())


    def frameCount(self):
        """
        The frame counter

        Returns
        ----------
        int
            Frame count
        """
        return lib.tof_data_frame_count(self._ptr, get_error())


    def frameType(self):
        """
        Get the frame type

        Returns
        ----------
        FrameType
            The frame type
        """
        return lib.tof_data_frame_type(self._ptr, get_error())


    def frameId(self):
        """
        Get the frame id

        Returns
        ----------
        int
            The frame id
        """
        return lib.tof_data_frame_id(self._ptr, get_error())


    def modulationFrequency(self):
        """
        Get the modulation frequency of the frame

        Returns
        ----------
        float
            The modulation frequency
        """
        return lib.tof_data_modulation_frequency(self._ptr, get_error())


    def integrationTime(self):
        """
        Get the integration time of the frame

        Returns
        ----------
        int
            The integration time
        """
        return lib.tof_data_integration_time(self._ptr, get_error())


    def process(self):
        """
        Get a bit array of the processing performed on the frame

        Returns
        ----------
        int
            A bit array
        """
        return lib.tof_data_process(self._ptr, get_error())


    def scale(self):
        """
        Get the scale used for integer frames, to get the original value divide by
        scale

        Returns
        ----------
        float
            The scale
        """
        return lib.tof_data_scale(self._ptr, get_error())


    def addition(self):
        """
        Get the addition used for integer frames, to get the original value
        substract the addition

        Returns
        ----------
        float
            The addition
        """
        return lib.tof_data_addition(self._ptr, get_error())


    def temperatures(self):
        """
        Get the temperature of various parts of the camera

        Returns
        ----------
        List[float]
            Temperatures
        """
        size = lib.tof_data_temperatures(self._ptr, ctypes.c_float(), 0, get_error())
        array = (ctypes.c_float * size)()
        lib.tof_data_temperatures(self._ptr, array, size, get_error())
        return array[:]


    def syncCount(self):
        """
        Get the synchronization count

        Returns
        ----------
        int
            Sync count
        """
        return lib.tof_data_sync_count(self._ptr, get_error())


    def userAllocated(self):
        """
        Check whether the data uses a pointer supplied by the user

        Returns
        ----------
        bool
            Data allocated by user
        """
        return lib.tof_data_user_allocated(self._ptr, get_error())


    def userPointer(self):
        """
        The user data that was given in the add_user_pointer function

        Returns
        ----------
        ctypes.c_void_p
            User pointer
        """
        return lib.tof_data_user_pointer(self._ptr, get_error())


    def acceleration(self):
        """
        The acceleration measured by the imu

        Returns
        ----------
        List[int]
            Acceleration, x y z
        """
        array = lib.tof_data_acceleration(self._ptr, get_error())
        return array.contents[:]


    def angularVelocity(self):
        """
        The angular velocity measured by the imu

        Returns
        ----------
        List[int]
            Angular velocity, x y z
        """
        array = lib.tof_data_angular_velocity(self._ptr, get_error())
        return array.contents[:]


# Custom code

    def __len__(self):
        return int(self.size())


    def __array__(self, dtype=None):
        """
        Use the Data class as a numpy array
        """
        import numpy as np

        def getShape(data):
            nchs = ((data.matType() & 0xF8) >> 3) + 1
            if nchs > 1:
                return (data.rows(), data.cols(), nchs)
            return (data.rows(), data.cols())

        def getDType(data):
            frame_type = np.uint32(data.matType()) & 0x0007
            if frame_type == 0:
                return np.dtype(np.uint8)
            elif frame_type == 1:
                return np.dtype(np.int8)
            elif frame_type == 2:
                return np.dtype(np.uint16)
            elif frame_type == 3:
                return np.dtype(np.int16)
            elif frame_type == 4:
                return np.dtype(np.int32)
            elif frame_type == 5:
                return np.dtype(np.float32)
            elif frame_type == 6:
                return np.dtype(np.float64)

        class ArrayDescriptor:
            def __init__(self, data):
                self.__array_interface__ = {
                    "shape": getShape(data),
                    "typestr": getDType(data).descr[0][1],
                    "data": (data.data(), False),
                    "version": 3,
                }

        if dtype == None:
            dtype = getDType(self)

        return np.array(ArrayDescriptor(self), dtype=dtype, copy=False)
